package org.jboss.resteasy.cdi.util;

import javax.ws.rs.core.MediaType;

/**
 * 
 * @author <a href="ron.sigal@jboss.com">Ron Sigal</a>
 * @version $Revision: 1.1 $
 *
 * Copyright Jun 29, 2012
 */
public class Constants
{
   public static final String MEDIA_TYPE_TEST_XML = "application/test+xml";
   public static final MediaType MEDIA_TYPE_TEST_XML_TYPE = MediaType.valueOf(MEDIA_TYPE_TEST_XML);
}

