@XmlSchema(xmlns = @XmlNs(namespaceURI = "http://www.w3.org/2005/Atom", prefix = "atom"))
package org.jboss.resteasy.links;

import javax.xml.bind.annotation.XmlNs;
import javax.xml.bind.annotation.XmlSchema;