package org.jboss.resteasy.jsapi.testing;
import javax.ws.rs.Path;

/**
 * 11 01 2012
 *
 * @author <a href="mailto:l.weinan@gmail.com">Weinan Li</a>
 */
@Path("/add")
public interface Add extends Operation {

}
