package org.jboss.resteasy.jose.jws;

/**
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Revision: 1 $
 */
public enum Algorithm
{
   none,
   HS256,
   HS384,
   HS512,
   RS256,
   RS384,
   RS512,
   ES256,
   ES384,
   ES512
}
