package org.jboss.resteasy.keystone.client;

/**
 * @author <a href="mailto:bill@burkecentral.com">Bill Burke</a>
 * @version $Revision: 1 $
 */
public interface TokensResource extends TokenFactory, TokenVerifier
{
}
