/**
 * This package (along with @see org.jboss.resteasy.test.cdi.events) tests CDI event processing
 * with JAX-RS components.
 * 
 * @see org.jboss.resteasy.package-info.java
*/
package org.jboss.resteasy.cdi.events;
