/**
 * This package (along with @see org.jboss.resteasy.test.cdi.decorators) tests the application
 * of CDI decorators to:
 * 
 * *) ContainerRequestFilter
 * *) ContainerResponseFilter
 * *) ReaderInterceptor
 * *) MessageBodyReader
 * *) WriterInterceptor
 * *) MessageBodyWriter
 * *) Resources
 * 
 * @see org.jboss.resteasy.package-info.java
 */
package org.jboss.resteasy.cdi.decorators;
